<?php
namespace Teknisa\FAI\Modules\Apuracao\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DTO\Response\Error;

class Fai04100CalculoRetroativos extends \Teknisa\GER\Controllers\TeknisaCrud{

    protected $util;
    protected $newCode;
    protected $crudService;
    protected $parameterBag;
    protected $connection;
    protected $environment;

    public function __construct($util, $newCode, $connection, $environment, $crudService){
        $this->util = $util;
        $this->newCode = $newCode;
        $this->connection = $connection;
        $this->environment = $environment;
        $this->crudService = $crudService;
    }


    public function insertList(DTO\Request\Row $request, DTO\Response $response){
        $row = $request->getRow();
        $nrlista = $this->util->insertLista($row);
        $response->addDataSet(new DataSet ('lista', array('nrlista' => $nrlista)));
    }
    
    public function populaTabelaTrab(DTO\Request\DataSet $request, DTO\Response $response){
        
        $rows = $request->getDataSet()->getRows()[0];
        $paramsProcedure = $request->getDataSet()->getRows()[1];
        try{
            $this->connection->beginTransaction();
            $novoCodigo = $this->newCode->getCodeByTableName('CTRLTRABRETROFAT', $this->environment->getNrorgTrab(), 1, 10, true);
            foreach($rows as $row){
                if ($row["VRUNAFATURAR"] > 0){
                    $pk = $this->newCode->getCodeByTableName('TRABRETROFAT', $this->environment->getNrorgTrab(), 1, 10, true);
                    $parametros = array(
                        "NRTRABRETROFAT" => $pk,
                        "NRCTRLTRABRETROFAT" => $novoCodigo,
                        "CDFILIAL" => $row["CDFILIAL"],
                        "CDSERVFILI" => $row["CDSERVFILI"],
                        "CDTURNO" => $row["CDTURNO"],
                        "NREFETIVOS" => $row["NREFETIVOS"],
                        "VRPRECUNIT" => $row["VRPRECUNIT"],
                        "VRPRECUNITATUAL" => $row["VRPRECUNITATUAL"],
                        "VRUNAFATURAR" => $row["VRUNAFATURAR"],
                        "NRGRUPONF" => null,
                        "__is_new" => true
                    );
                    try{
                        $dataSet = new DataSet('trabretrofat', array($parametros));
                        $request = new DTO\Request\DataSet($dataSet, '', '', '');
                        $rows = $this->crudService->getRowsToSave($request,'trabretrofat');
                        $this->crudService->save('trabretrofat', $rows);
                    }catch(\Exception $ex){
                        $response->setError(new Error("Nao foi possivel processar os dados enviados   --- " . $ex->getMessage(), 500));
                    }
                }
            } 
            $this->geraRetroativo($paramsProcedure, $novoCodigo);
            $this->connection->commit();
        }catch(\Exception $e){                
            $this->connection->rollback();
            throw $e;
        }

    }

    public function geraRetroativo($paramsProcedure, $novoCodigo){
            $nrorg = $this->environment->getNrorgTrab();
            $operador = $this->environment->getCdoperador();

            $procedure = new StoredProcedure($this->connection, 'GERA_RETROATIVO_GD');

            $procedure->addParam(new Param('P_CDFILIAL', Param::PARAM_INPUT, $paramsProcedure["CDFILIAL"]));
            $procedure->addParam(new Param('P_CDCLIENTE', Param::PARAM_INPUT, $paramsProcedure["CDCLIENTE"]));
            $procedure->addParam(new Param('P_LISTAAPURAC', Param::PARAM_INPUT, $paramsProcedure["LISTAAPURAC"]));
            $procedure->addParam(new Param('P_DTINICIAL', Param::PARAM_INPUT, $paramsProcedure["DTINICIAL"]));
            $procedure->addParam(new Param('P_DTFINAL', Param::PARAM_INPUT, $paramsProcedure["DTFINAL"]));
            $procedure->addParam(new Param('P_DTSAIDAPUR', Param::PARAM_INPUT, NULL));
            $procedure->addParam(new Param('P_NRCTRLTRABRETROFAT', Param::PARAM_INPUT, $novoCodigo));
            $procedure->addParam(new Param('P_CDOPERADOR', Param::PARAM_INPUT, $operador));
            $procedure->addParam(new Param('P_NRORG', Param::PARAM_INPUT, $nrorg));
            $procedure->addParam(new Param('P_CDTIPOOPER', Param::PARAM_INPUT, NULL));
            $procedure->addParam(new Param('P_NRLISTAERROS', Param::PARAM_OUTPUT, NULL));

            $procedure->execute();

    }

    //insere lista no banco
    public function insertLista($nrlista, $cdlista) {
        $params = array(
            "NRLISTA" => $nrlista,
            "CDLISTA" => $cdlista,
            "NRORG" => $this->environment->getNrorgTrab()
        );
        
        $sql = "INSERT INTO LISTA (NRLISTA, CDLISTA, NRORG) VALUES (:NRLISTA, :CDLISTA, :NRORG)";
        $query = $this->connection->prepare($sql);
        $query->execute($params);
    }
    
    //apaga lista do banco
    public function deleteLista($nrlista) {
        $params = array(
            "NRLISTA" => $nrlista
        );
        
        $sql = "DELETE FROM LISTA WHERE NRLISTA =:NRLISTA";
        $query = $this->connection->prepare($sql);
        $query->execute($params);
    }
    
    //cria lista de valores recebendo string como parametro
    public function criarListaValores(DTO\Request\Row $request, DTO\Response $response){
        $valores = $request->getRow()["valor"];
        if(strlen($valores) <= 4000){
            $response->addDataSet(new DataSet ('lista', array('nrlista' => $valores)));
        }else{
            $lista = explode('_',$valores);
            $nrlista = $this->newCode->getCodeByTableName('LISTA', $this->environment->getNrorgTrab(), 1, 10, false);
            try{
                foreach($lista as $key){
                    $this->insertLista($nrlista, $key);
                }
                
                $response->addDataSet(new DataSet ('lista', array('nrlista' => "_NRLISTA=".$nrlista)));
            } catch(\Exception $e){
                throw $e;
            }
        }
    }
    
    //apaga lista de valores recebendo string como parametro
    public function apagarListaValores(DTO\Request\Row $request, DTO\Response $response){
        $valores = $request->getRow()["lista"];
        foreach($valores as $valor){
            try{
                if(substr($valor, 0,9) == "_NRLISTA="){
                    $nrlista = substr($valor, 9);
                    $this->deleteLista($nrlista);
                }
            } catch(\Exception $e){
                throw $e;
            }
        }    
    }

    public function excluiRetroativo(DTO\Request\Row $request, DTO\Response $response){
        $paramsProcedure = $request->getRow();
        $nrorg = $this->environment->getNrorgTrab();
        $operador = $this->environment->getCdoperador();
        try{
            $this->connection->beginTransaction();
            $procedure = new StoredProcedure($this->connection, 'DELETE_RETROFAT');
            $procedure->addParam(new Param('P_NRRETROFAT', Param::PARAM_INPUT, $paramsProcedure["NRRETROFAT"], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDOPERADOR', Param::PARAM_INPUT, $operador, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRORG', Param::PARAM_INPUT, $nrorg, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_ERRO', Param::PARAM_OUTPUT, NULL, Param::PARAM_TYPE_STR, 30));
            $callback = $procedure->execute();   
            if($callback['P_ERRO'] == NULL){
                $this->connection->commit();
                $response->addDataSet(new DataSet ('callback', ['']));
            }else{
                throw new \Exception($callback['P_ERRO']);
            }            
        }catch(\Exception $e){                
            $this->connection->rollback();
            
            if(isset($callback) && $callback['P_ERRO'] != NULL){
                $response->addDataSet(new DataSet('callback', array($e->getMessage())));
            } else {
                $response->setError(new Error($e->getMessage(), 500));
            }
        }
    }

}