<?php
namespace Teknisa\FAI\Modules\Parametros\Controllers;

use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO;

class Fai31000ParametrosDeFaturamentoPorFilial extends \Teknisa\GER\Controllers\TeknisaCrud {

    const IDATIVO = 'S';
    protected $connection;
    protected $environment;
    protected $crudService;
    protected $parameterBag;
    protected $dataSourceName;
    protected $parameterBagColumns;
    protected $dataSourceManager;

    public function __construct($crudService, $parameterBag, $connection, $environment, $dataSourceManager) {
        $this->connection = $connection;
        $this->crudService  = $crudService;
        $this->parameterBag = $parameterBag;
        $this->environment  = $environment;
        $this->dataSourceManager  = $dataSourceManager;
        parent::__construct($crudService, $parameterBag);
    }

    public function removerParametrosDeFaturamento(DTO\Request\Row $request, DTO\Response $response) {
        $this->dataSourceName = "parfilialfat";
        $currentRow = $request->getRow();
        $dataSet = new DataSet($this->dataSourceName, array($currentRow));
        $this->dataSourceManager->delete($dataSet);
    }

    public function salvarParametrosDeFaturamento(DTO\Request\DataSet $request, DTO\Response $response) {
        try{
            $this->dataSourceName = "parfilialfat";
            $currentRow                   = $request->getDataSet()->getRows()[0];
            $currentTime                  = $this->environment->getCurrentTime()->format("d/m/Y H:i:s");
            $nrorgtrab                    = $this->environment->getNrorgTrab();
            $cdoperador                   = $this->environment->getCdoperador();
            $currentRow['DTINCLUSAO']     = $currentTime;
            $currentRow['DTULTATU']       = $currentTime;
            $currentRow['NRORGINCLUSAO']  = $nrorgtrab;
            $currentRow['CDOPERINCLUSAO'] = $cdoperador;
            $currentRow['NRORGULTATU']    = $nrorgtrab;
            $currentRow['VRPECREDICMFAT'] = str_replace(',','.',$currentRow['VRPECREDICMFAT']);
            $currentRow['CDOPERULTATU']   = $cdoperador;
            $currentRow['IDATIVO']        = self::IDATIVO;
            $dataSet                      = new DataSet($this->dataSourceName, array($currentRow));
            $this->dataSourceManager->persist($dataSet);
            $this->updateSerie($currentRow);
        }catch(\Exception $e){
            throw $e;
        }
    }


	private function updateSerie ($row){
        try{
    		$sql = "UPDATE FILIAL SET CDSERIEVANTECIPA = :P_CDSERIEVANTECIPA WHERE CDFILIAL = :P_CDFILIAL";
    				
    		$params = array(
    			"P_CDSERIEVANTECIPA" => $row['CDSERIEVANTECIPA'],
    			"P_CDFILIAL" => $row['CDFILIAL']
    		);
    		
    		$query = $this->connection->prepare($sql);
    		$query->execute($params);
        }catch(\Exception $e){
            throw $e;
        }
	}
    

    public function retornaParametrosDeFaturamento(DTO\Request\Filter $request, DTO\Response $response) {

        $this->dataSourceName = "fai31000_parfilialfat_filial";

        $this->parameterBagColumns = array("CDFILIAL", "NRORG");

        $filterCriteria = $request->getFilterCriteria();

        $conditions = $filterCriteria->getConditions();
        
        $cdfilial = $this->getFilterValueOf($conditions, 'CDFILIAL');
        if($cdfilial != 'T' && $cdfilial != '' && sizeof($cdfilial)>1) {
             $cdfilial = join(',', $cdfilial);
        } else if(sizeof($cdfilial)===1) {
            $cdfilial = $cdfilial[0];
        }

        $newConditions = array(
            array(
                "columnName" =>'CDFILIAL',
                "operator" => "IN",
                "value" => $cdfilial
            ),
            array(
                "columnName" =>'NRORG',
                "operator" => "=",
                "value" => $this->environment->getNrorgTrab()
            )
        );

        $filterCriteria = new FilterCriteria($this->dataSourceName, $newConditions, null, null);

        $newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria, $this->parameterBagColumns);

        $this->crudService->find($response, $newFilterCriteria);
    }
    
    public function getFilterValueOf($array, $columnName){
        $value = array_filter($array, function($element) use ($columnName){
        	return $element['columnName'] == $columnName;
        });
        
        $value = reset($value)['value'];
        
        return $value;
    }
}